@@ -4,6 +4,9 @@
 
 """Classes to help developers avoid code duplication when writing tests for pyiron."""
 
+from contextlib import redirect_stdout
+import doctest
+from io import StringIO
 import unittest
 from os.path import split, join
 from os import remove
@@ -11,6 +14,7 @@ from pyiron_base.project.generic import Project
 from abc import ABC
 from inspect import getfile
 
+
 __author__ = "Liam Huber"
 __copyright__ = (
     "Copyright 2020, Max-Planck-Institut für Eisenforschung GmbH - "
@@ -50,3 +54,28 @@ class TestWithCleanProject(TestWithProject, ABC):
 
     def tearDown(self):
         self.project.remove_jobs_silently(recursive=True)
+
+
+class TestWithDocstrings(unittest.TestCase, ABC):
+
+    """
+    Tests that also include testing the docstrings in the specified modules
+    """
+
+    @property
+    def docstring_module(self):
+        """
+        Define module whose docstrings will be tested
+        """
+        return None
+
+    def test_docstrings(self):
+        """
+        Fails with output if docstrings in the given module fails.
+
+        Output capturing adapted from https://stackoverflow.com/a/22434594/12332968
+        """
+        with StringIO() as buf, redirect_stdout(buf):
+            result = doctest.testmod(self.docstring_module)
+            output = buf.getvalue()
+        self.failIf(result.failed > 0, msg=output)
